@@ -2,8 +2,8 @@ import unittest
 import numpy as np
 
 import openmdao.api as om
-from openmdao.utils.assert_utils import assert_near_equal, assert_warning, assert_check_totals
-from openmdao.utils.om_warnings import  OMDeprecationWarning
+from openmdao.utils.assert_utils import assert_near_equal, assert_check_totals
+from openmdao.utils.testing_utils import use_tempdirs
 
 class Inner(om.Group):
     def setup(self):
@@ -23,7 +23,7 @@ class SrcIndicesTestCase(unittest.TestCase):
     def test_one_nesting(self):
         prob = om.Problem()
         model = prob.model
-        comp = model.add_subsystem('src', om.ExecComp('y=3*x', x=np.zeros((7)), y=np.zeros((7))))
+        model.add_subsystem('src', om.ExecComp('y=3*x', x=np.zeros((7)), y=np.zeros((7))))
         model.add_subsystem('outer', Outer())
         model.connect('src.y', 'outer.desvar_x', src_indices=[2, 4], flat_src_indices=True)
         prob.setup()
@@ -50,7 +50,7 @@ class SrcIndicesTestCase(unittest.TestCase):
 
         g1 = p.model.add_subsystem('g1', om.Group())
         # c2 is vectorized calculations
-        c2 = g1.add_subsystem('c2', om.ExecComp('z = a * y', shape=(4,)))
+        g1.add_subsystem('c2', om.ExecComp('z = a * y', shape=(4,)))
 
         # The ultimate source of a and y may be scalar, or have some other arbitrary shape
         g1.promotes('c2', inputs=['a'], src_indices=[0, 0, 0, 0], src_shape=(1,))
@@ -83,11 +83,11 @@ class SrcIndicesTestCase(unittest.TestCase):
 
         g1 = p.model.add_subsystem('g1', om.Group(), promotes_inputs=['b'])
         # c1 contains scalar calculations
-        c1 = g1.add_subsystem('c1', om.ExecComp('y = a0 + b', shape=(1,)),
-                              promotes_inputs=[('a0', 'a'), 'b'], promotes_outputs=['y'])
+        g1.add_subsystem('c1', om.ExecComp('y = a0 + b', shape=(1,)),
+                         promotes_inputs=[('a0', 'a'), 'b'], promotes_outputs=['y'])
         g2 = g1.add_subsystem('g2', om.Group())
         # c2 is vectorized calculations
-        c2 = g2.add_subsystem('c2', om.ExecComp('z = a * y', shape=(4,)), promotes_inputs=['a', 'y'])
+        g2.add_subsystem('c2', om.ExecComp('z = a * y', shape=(4,)), promotes_inputs=['a', 'y'])
 
         g1.promotes('g2', inputs=['y'], src_indices=[0, 0, 0, 0], src_shape=(1,))
         g1.promotes('g2', inputs=['a'], src_indices=[0, 0, 0, 0], src_shape=(1,))
@@ -106,14 +106,11 @@ class SrcIndicesTestCase(unittest.TestCase):
         assert_near_equal(p.model.get_val('a'), 99.)
         assert_near_equal(p.model.get_val('b'), 2.)
         assert_near_equal(p['g1.y'], 101.)
-        assert_near_equal(p['g1.a'], 99.)
         assert_near_equal(p.model.g1.get_val('y'), 101.)
         assert_near_equal(p.model.g1.get_val('a'), 99.)
         assert_near_equal(p['g1.c1.a0'], 99.)
         assert_near_equal(p['g1.c1.b'], 2.)
-        assert_near_equal(p['g1.g2.y'], [101.] * 4)
         assert_near_equal(p.model.g1.g2.get_val('y'), [101.] * 4)
-        assert_near_equal(p['g1.g2.a'], [99.] * 4)
         assert_near_equal(p.model.g1.g2.get_val('a'), [99.] * 4)
         assert_near_equal(p['g1.g2.c2.a'], [99.] * 4)
         assert_near_equal(p['g1.g2.c2.y'], [101.] * 4)
@@ -127,12 +124,12 @@ class SrcIndicesTestCase(unittest.TestCase):
 
         g1 = p.model.add_subsystem('g1', om.Group(), promotes_inputs=['b'])
         # c1 contains scalar calculations
-        c1 = g1.add_subsystem('c1', om.ExecComp('y = a0 + b', shape=(1,)),
-                              promotes_inputs=[('a0', 'a'), 'b'], promotes_outputs=['y'])
+        g1.add_subsystem('c1', om.ExecComp('y = a0 + b', shape=(1,)),
+                         promotes_inputs=[('a0', 'a'), 'b'], promotes_outputs=['y'])
 
         g2 = g1.add_subsystem('g2', om.Group())
         # c2 is vectorized calculations
-        c2 = g2.add_subsystem('c2',  om.ExecComp('z = a * y', shape=(4,)), promotes_inputs=['a', 'y'])
+        g2.add_subsystem('c2',  om.ExecComp('z = a * y', shape=(4,)), promotes_inputs=['a', 'y'])
 
         g1.promotes('g2', inputs=['a'], src_indices=[0, 0, 0, 0], src_shape=(1,))
         g1.promotes('g2', inputs=['y'], src_indices=[0, 0, 0, 0], src_shape=(1,))
@@ -316,6 +313,7 @@ class SrcIndicesTestCase(unittest.TestCase):
 
         with self.assertRaises(Exception) as cm:
             p.setup()
+            p.final_setup()
 
         self.assertEqual(cm.exception.args[0],
            "\nCollected errors for problem 'src_shape_mismatch':"
@@ -347,6 +345,7 @@ class SrcIndicesTestCase(unittest.TestCase):
         p.model.set_input_defaults('x', src_shape=src_shape)
         with self.assertRaises(Exception) as cm:
             p.setup()
+            p.final_setup()
 
         self.assertEqual(cm.exception.args[0],
             "\nCollected errors for problem 'src_indices_on_promotes':"
@@ -433,11 +432,12 @@ class SrcIndicesFeatureTestCase(unittest.TestCase):
         p.run_model()
 
         assert_near_equal(p['x'], inp)
-        assert_near_equal(p['G.x'], inp[:, :-1])
+        assert_near_equal(p.model.G.get_val('x'), inp[:, :-1])
         assert_near_equal(p['G.g1.C1.y'], inp[:, :-1][:, 1]*3.)
         assert_near_equal(p['G.g2.C2.y'], inp[:, :-1].flatten()[[1,5]]*2.)
 
 
+@use_tempdirs
 class SrcIndicesMPITestCase(unittest.TestCase):
     N_PROCS = 2
 
@@ -499,6 +499,7 @@ class SrcIndicesSerialMultipoint2(unittest.TestCase):
         p.run_model()
 
 
+@use_tempdirs
 class Multipoint2TooManyProcs(SrcIndicesSerialMultipoint2):
     N_PROCS = 3
 
@@ -529,14 +530,16 @@ class SrcIndicesSerialMultipoint3(unittest.TestCase):
         assert_check_totals(p.check_totals(of=['par.g1.C1.y', 'par.g2.C2.y', 'par.g3.C3.y'], wrt=['par.x']))
 
 
+@use_tempdirs
 class Multipoint3SameProcs(SrcIndicesSerialMultipoint3):
     N_PROCS = 3
 
 
+@use_tempdirs
 class Multipoint3TooManyProcs(SrcIndicesSerialMultipoint3):
     N_PROCS = 4
 
-
+@use_tempdirs
 class DoubleNestedParallelMultipointTestCase(unittest.TestCase):
     N_PROCS = 6
 
