@@ -1,16 +1,15 @@
 """
-A console script wrapper for multiple openmdao functions.
+OpenMDAO CLI functions.
 """
 
 import sys
 import os
 import argparse
-if sys.version_info.minor > 7:
-    import importlib.metadata as ilmd
-else:
-    ilmd = None
+import importlib.metadata as ilmd
 
 import re
+import subprocess
+
 from openmdao import __version__ as version
 
 try:
@@ -52,8 +51,11 @@ from openmdao.devtools.iprof_mem import _mem_prof_exec, _mem_prof_setup_parser,
     _mempost_exec, _mempost_setup_parser
 from openmdao.error_checking.check_config import _check_config_cmd, _check_config_setup_parser
 from openmdao.utils.mpi import MPI
+from openmdao.utils.file_utils import clean_outputs
 from openmdao.utils.find_cite import print_citations
 from openmdao.utils.code_utils import _calltree_setup_parser, _calltree_exec
+from openmdao.utils.jax_utils import _to_compute_primal_setup_parser, \
+    _to_compute_primal_exec
 from openmdao.utils.coloring import _total_coloring_setup_parser, _total_coloring_cmd, \
     _partial_coloring_setup_parser, _partial_coloring_cmd, \
     _view_coloring_setup_parser, _view_coloring_exec
@@ -61,9 +63,13 @@ from openmdao.utils.scaffold import _scaffold_setup_parser, _scaffold_exec
 from openmdao.utils.file_utils import _load_and_exec, _iter_entry_points
 from openmdao.utils.entry_points import _list_installed_setup_parser, _list_installed_cmd, \
     split_ep, _compute_entry_points_setup_parser, _compute_entry_points_exec, \
-        _find_plugins_setup_parser, _find_plugins_exec
+        _find_repos_setup_parser, _find_repos_exec
 from openmdao.utils.reports_system import _list_reports_setup_parser, _list_reports_cmd, \
     _view_reports_setup_parser, _view_reports_cmd
+from openmdao.visualization.graph_viewer import _graph_setup_parser, _graph_cmd
+from openmdao.visualization.realtime_opt_plot.realtime_opt_plot import \
+    _realtime_opt_plot_setup_parser, _realtime_opt_plot_cmd
+from openmdao.recorders.view_cases import _view_cases_setup_parser, _view_cases_cmd
 
 
 def _view_connections_setup_parser(parser):
@@ -106,22 +112,15 @@ def _view_connections_cmd(options, user_args):
         view_connections(prob, outfile=options.outfile, show_browser=not options.no_browser,
                          show_values=options.show_values, title=title)
 
-    # register the hook
-    if options.show_values:
-        funcname = 'final_setup'
-    else:
-        funcname = 'setup'
-
     def _view_model_w_errors(prob):
         if prob._metadata['saved_errors']:
-            # run the viewer here if we've had setup errors. Normally we'd wait until
-            # after setup or final_setup.
             _viewconns(prob)
             # errors will result in exit at the end of the _check_collected_errors method
 
+    # register the hooks
     hooks._register_hook('_check_collected_errors', 'Problem', pre=_view_model_w_errors)
-    hooks._register_hook(funcname, class_name='Problem', inst_id=options.problem, post=_viewconns,
-                         exit=True)
+    hooks._register_hook('final_setup', 'Problem', inst_id=options.problem,
+                         post=_viewconns, exit=True)
 
     _load_and_exec(options.file[0], user_args)
 
@@ -282,6 +281,45 @@ def _tree_setup_parser(parser):
                         help="Show which components compute approximations.")
 
 
+def _clean_setup_parser(parser):
+    """
+    Set up the openmdao subparser for the 'openmdao clean' command.
+
+    Parameters
+    ----------
+    parser : argparse subparser
+        The parser we're adding options to.
+    """
+    parser.add_argument('path', nargs='*', default='.',
+                        help='Path(s) from which OpenMDAO output directories should be removed.')
+    parser.add_argument('-f', '--noprompt', action='store_false', dest='prompt',
+                        help='Remove output directories without confirmation.')
+    parser.add_argument('-n', '--norecurse', action='store_true', dest='no_recurse',
+                        help='Do not recurse into subdirectories to find directories to remove.')
+    parser.add_argument('-d', '--dryrun', action='store_true', dest='dryrun',
+                        help='Highlight directories to be removed but do not actually remove them.')
+    parser.add_argument('-p', '--pattern', action='store', dest='pattern', default='*_out',
+                        help='Only directories whose name matches this glob pattern will be '
+                        'removed. This glob pattern applies to directory names found when '
+                        'recursing through the given paths. Surround this argument with quotation '
+                        'marks to prevent the OS from interpreting the glob pattern.')
+
+
+def _clean_cmd(options, user_args):
+    """
+    Return the post_setup hook function for 'openmdao summary'.
+
+    Parameters
+    ----------
+    options : argparse Namespace
+        Command line options.
+    user_args : list of str
+        Args to be passed to the user script.
+    """
+    clean_outputs(options.path, recurse=not options.no_recurse, prompt=options.prompt,
+                  pattern=options.pattern, dryrun=options.dryrun)
+
+
 def _get_tree_filter(attrs, vecvars):
     """
     Pull attributes and input/output vector variables out of a tree System.
@@ -399,7 +437,7 @@ def _cite_cmd(options, user_args):
         if not MPI or MPI.COMM_WORLD.rank == 0:
             print_citations(prob, classes=options.classes, out_stream=out)
 
-    hooks._register_hook('setup', 'Problem', post=_cite, exit=True)
+    hooks._register_hook('final_setup', 'Problem', post=_cite, exit=True)
     _load_and_exec(options.file[0], user_args)
 
 
@@ -439,6 +477,68 @@ def _list_pre_post_cmd(options, user_args):
     _load_and_exec(options.file[0], user_args)
 
 
+def _rtplot_setup_parser(parser):
+    """
+    Set up the openmdao subparser for the 'openmdao rtplot' command.
+
+    Parameters
+    ----------
+    parser : argparse subparser
+        The parser we're adding options to.
+    """
+    parser.add_argument('file', nargs=1, help='Python file containing the model.')
+    parser.add_argument('--no-display', action='store_false', dest='show',
+                        help="do not launch browser showing plot.")
+
+
+def _rtplot_cmd(options, user_args):
+    """
+    Return the post_setup hook function for 'openmdao list_pre_post'.
+
+    Parameters
+    ----------
+    options : argparse Namespace
+        Command line options.
+    user_args : list of str
+        Args to be passed to the user script.
+    """
+
+    def _view_realtime_opt_plot(problem):
+        driver = problem.driver
+        if not driver:
+            raise RuntimeError(
+                "Unable to run realtime optimization progress plot because no Driver")
+        if len(problem.driver._rec_mgr._recorders) == 0:
+            raise RuntimeError(
+                "Unable to run realtime optimization progress plot "
+                    "because no case recorder attached to Driver"
+            )
+
+        recorder_filepath = str(problem.driver._rec_mgr._recorders[0]._filepath)
+
+        if options.show:
+            cmd = ['openmdao', 'realtime_opt_plot', '--pid', str(os.getpid()), recorder_filepath]
+        else:
+            cmd = ['openmdao', 'realtime_opt_plot', '--pid', '--no-display',
+                   str(os.getpid()), recorder_filepath]
+        cp = subprocess.Popen(cmd)  # nosec: trusted input
+
+        # Do a quick non-blocking check to see if it immediately failed
+        # This will catch immediate failures but won't wait for the process to finish
+        quick_check = cp.poll()
+        if quick_check is not None and quick_check != 0:
+            # Process already terminated with an error
+            stderr = cp.stderr.read().decode()
+            raise RuntimeError(
+                f"Failed to start up the realtime plot server with code {quick_check}: {stderr}.")
+
+    # register the hook
+    hooks._register_hook('_setup_recording', 'Problem', post=_view_realtime_opt_plot, ncalls=1)
+
+    # run the script
+    _load_and_exec(options.file[0], user_args)
+
+
 def _get_deps(dep_dict: dict, package_name: str) -> None:
     """
     Recursively determine all installed dependency versions and add newly found ones to dep_dict.
@@ -527,57 +627,175 @@ def _comm_info_cmd(options, user_args):
 # this dict should contain names mapped to tuples of the form:
 #   (setup_parser_func, executor, description)
 _command_map = {
-    'call_tree': (_calltree_setup_parser, _calltree_exec,
-                  "Display the call tree for the specified class method and all 'self' class "
-                  "methods it calls."),
-    'check': (_check_config_setup_parser, _check_config_cmd,
-              'Perform a number of configuration checks on the problem.'),
-    'cite': (_cite_setup_parser, _cite_cmd, 'Print citations referenced by the problem.'),
-    'comm_info': (_comm_info_setup_parser, _comm_info_cmd,
-                  'Print MPI communicator info for systems.'),
-    'compute_entry_points': (_compute_entry_points_setup_parser, _compute_entry_points_exec,
-                             'Compute entry point declarations to add to the setup.py file.'),
-    'dist_conns': (_dist_conns_setup_parser, _dist_conns_cmd,
-                   'Display connection information for variables across multiple MPI processes.'),
-    'find_plugins': (_find_plugins_setup_parser, _find_plugins_exec,
-                     'Find openmdao plugins on github.'),
-    'iprof': (_iprof_setup_parser, _iprof_exec,
-              'Profile calls to particular object instances.'),
-    'iprof_totals': (_iprof_totals_setup_parser, _iprof_totals_exec,
-                     'Generate total timings of calls to particular object instances.'),
-    'list_installed': (_list_installed_setup_parser, _list_installed_cmd,
-                       'List installed types recognized by OpenMDAO.'),
-    'list_reports': (_list_reports_setup_parser, _list_reports_cmd,
-                     'List available reports.'),
-    'mem': (_mem_prof_setup_parser, _mem_prof_exec,
-            'Profile memory used by OpenMDAO related functions.'),
-    'mempost': (_mempost_setup_parser, _mempost_exec, 'Post-process memory profile output.'),
-    'n2': (_n2_setup_parser, _n2_cmd, 'Display an interactive N2 diagram of the problem.'),
-    'partial_coloring': (_partial_coloring_setup_parser, _partial_coloring_cmd,
-                         'Compute coloring(s) for specified partial jacobians.'),
-    'scaffold': (_scaffold_setup_parser, _scaffold_exec,
-                 'Generate a simple scaffold for a component.'),
-    'scaling': (_scaling_setup_parser, _scaling_cmd, 'View driver scaling report.'),
-    'list_pre_post': (_list_pre_post_setup_parser, _list_pre_post_cmd,
-                      'Show pre and post setup systems.'),
-    'summary': (_config_summary_setup_parser, _config_summary_cmd,
-                'Print a short top-level summary of the problem.'),
-    'timing': (_timing_setup_parser, _timing_cmd, 'Collect timing information for all systems.'),
-    'total_coloring': (_total_coloring_setup_parser, _total_coloring_cmd,
-                       'Compute a coloring for the total jacobian.'),
-    'trace': (_itrace_setup_parser, _itrace_exec, 'Dump trace output.'),
-    'tree': (_tree_setup_parser, _tree_cmd, 'Print the system tree.'),
-    'view_coloring': (_view_coloring_setup_parser, _view_coloring_exec, 'View a colored jacobian.'),
-    'view_connections': (_view_connections_setup_parser, _view_connections_cmd,
-                         'View connections showing values and source/target units.'),
-    'view_dyn_shapes': (_view_dyn_shapes_setup_parser, _view_dyn_shapes_cmd,
-                        'View the dynamic shape dependency graph.'),
-    'view_mm': (_meta_model_parser, _meta_model_cmd, "View a metamodel."),
-    'view_reports': (_view_reports_setup_parser, _view_reports_cmd,
-                     'View existing reports.'),
+    "call_tree": (
+        _calltree_setup_parser,
+        _calltree_exec,
+        "Display the call tree for the specified class method and all 'self' class "
+        "methods it calls.",
+    ),
+    "check": (
+        _check_config_setup_parser,
+        _check_config_cmd,
+        "Perform a number of configuration checks on the problem.",
+    ),
+    "cite": (
+        _cite_setup_parser,
+        _cite_cmd,
+        "Print citations referenced by the problem.",
+    ),
+    "clean": (_clean_setup_parser, _clean_cmd, "Remove OpenMDAO output directories."),
+    "comm_info": (
+        _comm_info_setup_parser,
+        _comm_info_cmd,
+        "Print MPI communicator info for systems.",
+    ),
+    "compute_entry_points": (
+        _compute_entry_points_setup_parser,
+        _compute_entry_points_exec,
+        "Compute entry point declarations to add to the setup.py file.",
+    ),
+    "dist_conns": (
+        _dist_conns_setup_parser,
+        _dist_conns_cmd,
+        "Display connection information for variables across multiple MPI processes.",
+    ),
+    "find_repos": (
+        _find_repos_setup_parser,
+        _find_repos_exec,
+        "Find repos on github having openmdao topics.",
+    ),
+    "graph": (_graph_setup_parser, _graph_cmd, "Generate a graph for a group."),
+    "iprof": (
+        _iprof_setup_parser,
+        _iprof_exec,
+        "Profile calls to particular object instances.",
+    ),
+    "iprof_totals": (
+        _iprof_totals_setup_parser,
+        _iprof_totals_exec,
+        "Generate total timings of calls to particular object instances.",
+    ),
+    "list_installed": (
+        _list_installed_setup_parser,
+        _list_installed_cmd,
+        "List installed types recognized by OpenMDAO.",
+    ),
+    "list_pre_post": (
+        _list_pre_post_setup_parser,
+        _list_pre_post_cmd,
+        "Show pre and post setup systems.",
+    ),
+    "list_reports": (
+        _list_reports_setup_parser,
+        _list_reports_cmd,
+        "List available reports.",
+    ),
+    "mem": (
+        _mem_prof_setup_parser,
+        _mem_prof_exec,
+        "Profile memory used by OpenMDAO related functions.",
+    ),
+    "mempost": (
+        _mempost_setup_parser,
+        _mempost_exec,
+        "Post-process memory profile output.",
+    ),
+    "n2": (
+        _n2_setup_parser,
+        _n2_cmd,
+        "Display an interactive N2 diagram of the problem.",
+    ),
+    "partial_coloring": (
+        _partial_coloring_setup_parser,
+        _partial_coloring_cmd,
+        "Compute coloring(s) for specified partial jacobians.",
+    ),
+    'rtplot': (
+        _rtplot_setup_parser,
+        _rtplot_cmd,
+        "Run the realtime optimization progress plot tool once the driver recorder file is started"
+    ),
+    'realtime_opt_plot': (
+        _realtime_opt_plot_setup_parser,
+        _realtime_opt_plot_cmd,
+        "Run the realtime optimization progress plot tool"
+    ),
+    "scaffold": (
+        _scaffold_setup_parser,
+        _scaffold_exec,
+        "Generate a simple scaffold for a component.",
+    ),
+    "scaling": (_scaling_setup_parser, _scaling_cmd, "View driver scaling report."),
+    "summary": (
+        _config_summary_setup_parser,
+        _config_summary_cmd,
+        "Print a short top-level summary of the problem.",
+    ),
+    "timing": (
+        _timing_setup_parser,
+        _timing_cmd,
+        "Collect timing information for all systems.",
+    ),
+    "to_compute_primal": (
+        _to_compute_primal_setup_parser,
+        _to_compute_primal_exec,
+        "Convert a component to use compute_primal instead of compute or "
+        "apply_nonlinear.",
+    ),
+    "total_coloring": (
+        _total_coloring_setup_parser,
+        _total_coloring_cmd,
+        "Compute a coloring for the total jacobian.",
+    ),
+    "trace": (_itrace_setup_parser, _itrace_exec, "Dump trace output."),
+    "tree": (_tree_setup_parser, _tree_cmd, "Print the system tree."),
+    "view_cases": (
+        _view_cases_setup_parser,
+        _view_cases_cmd,
+        "View a case recorder file.",
+    ),
+    "view_coloring": (
+        _view_coloring_setup_parser,
+        _view_coloring_exec,
+        "View a colored jacobian.",
+    ),
+    "view_connections": (
+        _view_connections_setup_parser,
+        _view_connections_cmd,
+        "View connections showing values and source/target units.",
+    ),
+    "view_dyn_shapes": (
+        _view_dyn_shapes_setup_parser,
+        _view_dyn_shapes_cmd,
+        "View the dynamic shape dependency graph.",
+    ),
+    "view_mm": (_meta_model_parser, _meta_model_cmd, "View a metamodel."),
+    "view_reports": (
+        _view_reports_setup_parser,
+        _view_reports_cmd,
+        "View existing reports.",
+    ),
 }
 
 
+def _register_view_reports():
+    """
+    Set hook to view reports after running an openmdao script.
+    """
+    hooks.use_hooks = True
+
+    # request cleanup at exit
+    om_atexit = os.environ.get('OPENMDAO_ATEXIT')
+    os.environ['OPENMDAO_ATEXIT'] = ','.join([om_atexit, 'cleanup']) if om_atexit else 'cleanup'
+
+    from openmdao.utils.reports_system import view_reports
+
+    def run_problem_reports(problem):
+        view_reports(problem._name)
+
+    hooks._register_hook('cleanup', 'Problem', post=run_problem_reports, exit=False)
+
+
 def openmdao_cmd():
     """
     Run an 'openmdao' sub-command or list help info for 'openmdao' command or sub-commands.
@@ -599,10 +817,14 @@ def openmdao_cmd():
                                      'For example: '
                                      '"openmdao n2 -o foo.html myscript.py -- -x --myarg=bar"')
 
-    ver_group = parser.add_mutually_exclusive_group()
-    ver_group.add_argument('--version', action='version', version=version)
-    ver_group.add_argument('--dependency_versions', action='store_true', default=False,
+    opt_group = parser.add_mutually_exclusive_group()
+    opt_group.add_argument('--version', action='version', version=version)
+    opt_group.add_argument('--dependency_versions', action='store_true', default=False,
                            help="show versions of OpenMDAO and all dependencies, then exit")
+    opt_group.add_argument('--view_reports', action='store_true', default=False,
+                           help="after running an OpenMDAO script, display any generated reports")
+
+    opts = ('-h', '--help', '--version', '--dependency_versions', '--view_reports')
 
     # setting 'dest' here will populate the Namespace with the active subparser name
     subs = parser.add_subparsers(title='Tools', metavar='', dest="subparser_name")
@@ -636,10 +858,13 @@ def openmdao_cmd():
         subp.set_defaults(executor=executor)
 
     args = [a for a in sys.argv[1:] if not a.startswith('-')]
-    cmdargs = [a for a in sys.argv[1:] if a not in ('-h', '--version', '--dependency_versions')]
+    cmdopts = [a for a in sys.argv[1:] if a in opts]
+    cmdargs = [a for a in sys.argv[1:] if a not in opts]
 
     # handle case where someone just runs `openmdao <script> [dashed-args]`
     if not set(args).intersection(subs.choices) and len(args) == 1 and os.path.isfile(cmdargs[0]):
+        if '--view_reports' in cmdopts:
+            _register_view_reports()
         _load_and_exec(args[0], user_args)
     else:
         hooks.use_hooks = True
@@ -660,7 +885,7 @@ def openmdao_cmd():
 
         if hasattr(options, 'executor'):
             options.executor(options, user_args)
-        elif options.dependency_versions is True and ilmd is not None:
+        elif options.dependency_versions is True:
             dep_versions = {}
             _get_deps(dep_versions, 'openmdao')
 
